
#include "LL_USB_big.h"
#include "stdio.h"
#include <termios.h>


BOOLEAN fDacSet = FALSE;
BOOLEAN fDigoutSent = FALSE;


BYTE bRetVal = 0;
BYTE bLastDAC = 0;
BYTE bMainChan = 0;
BYTE bPollChan = 0;
BYTE bLastDigout = 0;
double dblVolts = 3.5;
double dblLastDAC_Val = 0;

//###############
//###############
//###############
//## MAIN LOOP ##
//###############
BYTE bNowDig = 0;
#include <termcap.h>
static char termbuf[2048];
int main() 
{	//NOTE: default var vals must be set here to share common header
	// could set these in a varSet() type function

	AllowPrintf(TRUE);

	fCurrentScanTypeChanged = FALSE;
	dblRate = 100;
	dblDataRate = 100;
	bSingleChanScanChan = 0;
	fSignedOn = FALSE;
	fGotDigin = FALSE;
	bLastDigin = 0;
	fDoScanLog = FALSE;
	fDoDispData = TRUE; 

	usDeviceID = 0;
	iErr = 0;
	bDBG = 0;
	fContScanDone = FALSE;		
	fScanThreadActive = FALSE;	


	//following just defaults, will be asked for both when the scan is started
	//search "doSetNumberOfScans" for explanation using each of the following
	fContinuousScan = FALSE;	//this trumps the others. If true, uiScansToDo is ignored
	fTimedScan = FALSE;
	uiScansToDo = 100;		//"scans" not data points (default to 100)
	uiScanHours = 1;		//just any default
	uiScanMinutes = 1;		//just any default 
	uiScanSeconds = 60;		//just any default 
	fScanHours = FALSE;		//just any default 
	fScanMinutes = FALSE;	//just any default 
	fScanSeconds = FALSE;	//just any default 


	fPollOrScanMenuShowing = POLL_MENU;
	scanStruct.uiScanToken = CMND_SINGLE_CHAN_SCAN;
	scanStruct.uiScanChans = 1;
	scanStruct.fGotBuffWrap = FALSE;
	scanStruct.fScanThreadLoopDone = FALSE;
	scanStruct.isScanning = FALSE; 
	scanStruct.inThread = 0;
	scanStruct.uiStackedScans = 1; //MUST ALWAYS BE AT LEAST 1
	scanStruct.uiHeadPntr = 0;
	scanStruct.uiTailPntr = 0;


	//from: http://stackoverflow.com/questions/1022957/getting-terminal-width-in-c
    char *termtype = getenv("TERM");
	fConsoleSizeChanged = FALSE;

	//the following attempts to resize the terminal so it's wide/tall enough to accommodate all the menu
	// items and also display data from 8 channels when displaying data in multi-chan-cal-scan mode
    if (tgetent(termbuf, termtype) < 0) 
	{
        //error(EXIT_FAILURE, 0, "Could not access the termcap data base.\n");
		int cInput;
		system ("clear");
		printf("\n\nCannot determine your console size (rows and columns)  \n");   
		printf(  "\nThis app requires minimum 25 rows and 80 columns for\n");   
		printf(  "\n  it to display properly  \n\n");
		
		printf("Should I continue, or allow you to resize and try again E)xit, C)ontinue: ");
		while(chkKbHit() == TRUE) { usleep(10); doGetch();}
		cInput = doGetch();
		if( ((char)cInput != 'c') && ((char)cInput != 'C') )
		{
			system ("clear");
			exit(0);
		}
    }
	else
	{
		uiConsoleStartHeight = tgetnum("li"); 
		uiConsoleStartWidth = tgetnum("co");
		if((uiConsoleStartHeight < 25) || (uiConsoleStartWidth < 120))
		{
			system ("clear");
			printf("\n\nYour current console size %d x %d will not display app correctly.\n\n", uiConsoleStartWidth, uiConsoleStartHeight);
			printf("Should I resize it to 125 x 25 for proper app display? \n  It will be returned to original size when app exits. Y)es, N)o: ");
			while(chkKbHit() == TRUE) { usleep(10); doGetch();}
			int cInput;
			cInput = doGetch();

			if( ((char)cInput == 'y') || ((char)cInput == 'Y') )
			{
				system ("clear");
				printf("\n\nWill attempt to reset console size.\n");
				printf("  note that it may not work on every system");;
				printf ("\e[8;25;125;t");
				fConsoleSizeChanged = TRUE;
			}
			else
			{
				if((uiConsoleStartHeight < 25) || (uiConsoleStartWidth < 80))
				{
					system ("clear");
					printf("\n\nYou must manually resize console window to show a minimum of 25  \n"   
						       "  lines  and 80 columns for all of app's menu items to be displayed\n\n");
					printf("any key to exit");;
					while(chkKbHit() == TRUE) { usleep(10); doGetch();}
					doGetch();
					system ("clear");
					exit(0);
				}
			}
		}
	}

	// from http://www.commandlinefu.com/commands/view/4509/resize-a-terminal-window
	//Resize a Terminal Window
	//Replace 70 with the desired height.
	//Replace 180 with the desired width.
	//Example: printf ("\e[8;70;40;t");
	//	printf ("\e[8;25;125;t");
	//	printf ("\x1b[7h");


	int cInput;
	initscr();
	refresh();
	cbreak();

	int newWinX = 1;
	int newWinY = 1;
	if(fConsoleSizeChanged)
	{ newWinX = 20; }

	win_body = newwin(25, 120, 0, newWinX);  //height, width, y, x

	//nodelay(win_body, TRUE);
	box(win_body, ' ', ' '); //frame (corners only shown) around the actual window area
	//don't wait for key input with wgetch so program can continue with kbhit()
	// change to FALSE to actually wait if needed
	keypad(win_body, TRUE); 
	curs_set(0);
	wmove(win_body, 24, 1);

	showMainMenu();

	wrefresh(win_body);
	noecho();	//no echo key input

	//following just loops until program exits, processing keystrokes from either 
	// the polled, or scan menu screens
	while(TRUE)
	{
		while(!kbhit(FALSE)) 
		{
			//note that this call also checks to see if device is connected.
		}
		cInput = wgetch(win_body);
		char upChar = toupper((char)cInput);
		
		if(fPollOrScanMenuShowing == POLL_MENU)
		{
			switch(upChar)
			{
				case 'D':
					wmove(win_body, 24, 1); wclrtoeol(win_body);
					USHORT usDevID_temp = 0;
					chkDevID(&usDevID_temp);
					break;

				case 'R':
					doRate();
					break;

				case 'C':
					doChan();
					break;

				case 'A':
					doSetDAC();
					break;

				case 'O':
					sendDigout(); 
					break;

				case 'I':
					getDigin();  
					break;

				case 'G':
					getPolledModeVoltage();
					break;

				case 'M':  
					getMiscFuncs();
					break;

				case 'S':
					if(!fSignedOn) 
					{
						wmove(win_body, 24, 1); wclrtoeol(win_body);
						sprintf(scrnBuff, "Must set a device ID first"); 
						mvwaddstr(win_body, 24, 1, scrnBuff);	
						wrefresh(win_body);
						break;
					}

					wclear(win_body); showScanMenu(); 
					break;

				case 'Q':

					cleanupForExit();  //add any cleanup code - nothing yet
					curs_set(1); echo(); endwin(); 
					system("clear");
					if(fConsoleSizeChanged)
					{
						char tempBuff[64];
						//printf ("\e[8;25;125;t");
						sprintf(tempBuff, "\e[8;%d;%d;t", uiConsoleStartHeight, uiConsoleStartWidth);  
						printf ("%s", tempBuff);
						fConsoleSizeChanged = TRUE;
					}
					return 1;
					break;

				default:
					showMainMenu();  
					break;
			}
		}
		else
		{
			switch(upChar)
			{
				case 'T':
					doSetScanType();
					break;

				case 'N':
					//number of scans can be either continuous, which waits for keystroke,
					// timed, based on certain amount of time passed, or count, based on
					// number of scans. Any scan will exit with a keypress though. The
					// default is to scan until a key is pressed.
					doSetNumberOfScans();  
					break;

				case 'C':
					doSetScanChans();
					break;

				case 'R':
					doRate();
					break;

				case 'D':
					doSetScanLog();
					break;

				case 'S':
					//set fContinuousScan or uiScansToDo FIRST
					doStartScan();
					break;

				case 'E':
					wclear(win_body);
					wrefresh(win_body);
					showMainMenu(); 
					break;

				default:
					break;
			}
		}
	}
	return 1; //all is done in main loop above
}

//#################
//#################
//#################
//## SEND DIGOUT ##
//#################
sendDigout()
{
	if(!fSignedOn) notSignedOnCheck();
	if(!fSignedOn) return FALSE;
	BYTE bDigout_;

	char tbuff[32]; memset(tbuff, 'x', sizeof(tbuff)); //set to non-zero value
	curs_set(1);
	echo();
	keypad(win_body, FALSE); 
	while(TRUE)
	{
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Enter digout 0-255: ");
		mvwaddstr(win_body, 24, 1, scrnBuff);	
		mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
		bDigout_ = (BYTE)atoi(tbuff);
		if(bDigout_ <= 255) break;
	}
	sprintf(scrnBuff, "Sending digital output: %u - WAIT", bDigout_);
	mvwaddstr(win_body, 24, 1, scrnBuff);
	bDBG = 0;
	wrefresh(win_body);
	wmove(win_body, 24, 1); wclrtoeol(win_body);
	//BYTE setDigout(struct libusb_device_handle *hDevice, USHORT usDevID, BYTE bDigout, BYTE bDBG, int* iErr);
	if(setDigout(hDevice_global, usDeviceID, bDigout_, bDBG, &iErr))
	{
		bLastDigout = bDigout_;
		fDigoutSent = TRUE;
		curs_set(0);
		noecho();
		keypad(win_body, TRUE); 
		wrefresh(win_body);
		bLastDigout = bDigout_;
		wmove(win_body, 11, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "O) Send a digital output (last: %3d)    ", bLastDigout);
		putMainScrn(11, scrnBuff);
		sprintf(scrnBuff, "Digital output: %u send success", bLastDigout);
		mvwaddstr(win_body, 24, 1, scrnBuff);
		wrefresh(win_body);
	}
	else
	{
		sprintf(scrnBuff, "Digital output: %u send FAILED", bDigout_);
		mvwaddstr(win_body, 24, 1, scrnBuff);
	}
	curs_set(0);
	noecho();
	keypad(win_body, TRUE); 
	return TRUE;
}

//#################
//#################
//#################
//## SET DAC ######
//#################
BOOLEAN doSetDAC(void)
{
	if(!fSignedOn) notSignedOnCheck();
	if(!fSignedOn) return FALSE;
	BYTE bLastDAC_;
	DOUBLE dblLastDAC_Val_;
	char tbuff[32]; memset(tbuff, 'x', sizeof(tbuff)); //set to non-zero value

	curs_set(1);
	echo();
	keypad(win_body, FALSE); 
	while(TRUE)
	{
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Enter DAC (0, 1, or 2): ");
		mvwaddstr(win_body, 24, 1, scrnBuff);	
		mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
		bLastDAC_ = (BYTE)atoi(tbuff);
		if(bLastDAC_ <= 3) break;
	}
	while(TRUE)
	{
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Enter DAC volts (-5 to 5): ");
		mvwaddstr(win_body, 24, 1, scrnBuff);	
		mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, 5);
		//mvwgetstr(win_body, tbuff);
		dblLastDAC_Val_ = (DOUBLE)atof(tbuff);
		if((dblLastDAC_Val_ >= (-5)) && (dblLastDAC_Val_ <= (5))) break;
	}

	wmove(win_body, 24, 1); wclrtoeol(win_body);
	sprintf(scrnBuff, "Sending %1.2lf to DAC: %d - WAIT", dblLastDAC_Val_, bLastDAC_);
	mvwaddstr(win_body, 24, 1, scrnBuff);
	bDBG = 0;
	wrefresh(win_body);
	wmove(win_body, 24, 1); wclrtoeol(win_body);
	if(setDAC(hDevice_global, usDeviceID, bLastDAC_, &dblLastDAC_Val_, bDBG, &iErr))
	{
		dblLastDAC_Val = dblLastDAC_Val_;
		bLastDAC = bLastDAC_;
		fDacSet = TRUE;
		curs_set(0);
		noecho();
		keypad(win_body, TRUE); 
		wrefresh(win_body);
		wmove(win_body, 9, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "A) Set DAC voltage (last: DAC: %d volts: %1.3lf)", bLastDAC, dblLastDAC_Val);
		mvwaddstr(win_body, 9, ((80 / 2) - (strlen(scrnBuff) / 2)) + 3, scrnBuff);

		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Set DAC: %d volts: %1.3lf) ", bLastDAC, dblLastDAC_Val);
		mvwaddstr(win_body, 24, 1, scrnBuff);
		wrefresh(win_body);
	}
	else
	{
		sprintf(scrnBuff, "DAC output FAILED");
		mvwaddstr(win_body, 24, 1, scrnBuff);
	}
	curs_set(0);
	noecho();
	keypad(win_body, TRUE); 
	return TRUE;
}

//#################
//#################
//#################
//## CHAN CHANGE ##
//#################
BOOLEAN doChan()
{
	if(!fSignedOn) notSignedOnCheck();
	if(!fSignedOn) return FALSE;
	BYTE bChanTemp;

	char tbuff[32]; memset(tbuff, 'x', sizeof(tbuff)); //set to non-zero value
	curs_set(1);
	echo();
	keypad(win_body, FALSE); 
	while(TRUE)
	{
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Enter channel 0-7: ");
		mvwaddstr(win_body, 24, 1, scrnBuff);	
		mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
		bChanTemp = (BYTE)atoi(tbuff);
		if(bChanTemp < 8) break;
	}
	sprintf(scrnBuff, "Sending channel: %u - WAIT", bChanTemp);
	mvwaddstr(win_body, 24, 1, scrnBuff);
	bDBG = 0;
	wrefresh(win_body);
	wmove(win_body, 24, 1); wclrtoeol(win_body);
	if(sendChan(hDevice_global, usDeviceID, bChanTemp, bDBG, &iErr))
	{
		curs_set(0);
		noecho();
		keypad(win_body, TRUE); 
		wrefresh(win_body);
		bSingleChanScanChan = bChanTemp;
		wmove(win_body, 7, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "C) Enter a channel (current: %d)         ", bSingleChanScanChan);
		putMainScrn(7, scrnBuff);
		sprintf(scrnBuff, "Channel: %u send success", bChanTemp);
		mvwaddstr(win_body, 24, 1, scrnBuff);
		wrefresh(win_body);
	}
	else
	{
		sprintf(scrnBuff, "Channel: %u send FAILED", bChanTemp);
	}
	mvwaddstr(win_body, 24, 1, scrnBuff);
	curs_set(0);
	noecho();
	keypad(win_body, TRUE); 
	return TRUE;
}

//##################################
//##################################
//##################################
//## DOSETSCANLOG ##################
//##################################
BOOLEAN doSetScanLog(void)
{
	curs_set(1);
	echo();
	keypad(win_body, FALSE); 

	wmove(win_body, 24, 1); wclrtoeol(win_body);
	//sprintf(scrnBuff, "Log to (F)ile, (D)isplay, (B)oth: ");
	sprintf(scrnBuff, "Log to (F)ile, (D)isplay, (B)oth, (N)one: ");
	mvwaddstr(win_body, 24, 1, scrnBuff);	
	int cInput = wgetch(win_body);
	char upChar = toupper((char)cInput);
	wmove(win_body, 24, 1); wclrtoeol(win_body);
	if(upChar == 'F')
	{
		fDoScanLog = TRUE; fDoDispData = FALSE;
		//sprintf(scrnBuff, "will log to file: [datestamp].txt but not display on screen");
	}
	if(upChar == 'D')
	{
		fDoScanLog = FALSE; fDoDispData = TRUE;
		//sprintf(scrnBuff, "will display on screen but not log to file");
	}
	if(upChar == 'B')
	{
		fDoScanLog = TRUE; fDoDispData = TRUE;
		//sprintf(scrnBuff, "will log to file: [datestamp].txt and display on screen");
	}
	if(upChar == 'N')
	{
		fDoScanLog = FALSE; fDoDispData = FALSE;
		//sprintf(scrnBuff, "will log to file: [datestamp].txt and display on screen");
	}

	curs_set(0);
	noecho();
	keypad(win_body, TRUE);
	showScanMenu();
	return TRUE;	
}

//##################################
//##################################
//##################################
//## GETMISCFUNCS ##################
//##################################
BOOLEAN getMiscFuncs(void)
{
	if(!fSignedOn) notSignedOnCheck();
	if(!fSignedOn) return FALSE;

	curs_set(1);
	echo();
	keypad(win_body, FALSE); 

	wmove(win_body, 24, 1); wclrtoeol(win_body);
	sprintf(scrnBuff, "(S)ystem cal, (F)ullscale cal, (O)ffset cal: ");
	mvwaddstr(win_body, 24, 1, scrnBuff);	

	int cInput = wgetch(win_body);
	char upChar = toupper((char)cInput);
	wmove(win_body, 24, 1); wclrtoeol(win_body);

	BYTE bDBG = 0;
	int iErr = 0;

	switch(upChar)
	{
		case 'O':
			sprintf(scrnBuff, "doing offset calibration. Wait . . .");
			mvwaddstr(win_body, 24, 1, scrnBuff);
			wrefresh(win_body);
			doOffsetCal(hDevice_global, usDeviceID, bDBG, &iErr);
			break;
		case 'F':
			sprintf(scrnBuff, "doing full-scale calibration. Wait . . .");
			mvwaddstr(win_body, 24, 1, scrnBuff);	
			wrefresh(win_body);
			doFullscaleCal(hDevice_global, usDeviceID, bDBG, &iErr);
			break;
		case 'S':
			sprintf(scrnBuff, "doing system calibration. Wait . . .");
			mvwaddstr(win_body, 24, 1, scrnBuff);	
			wrefresh(win_body);
			doSyscal(hDevice_global, usDeviceID, bDBG, &iErr);
			break;
		default:
			break;
	}
		
	wmove(win_body, 24, 1); wclrtoeol(win_body);
	curs_set(0);
	noecho();
	keypad(win_body, TRUE);

	showMainMenu();
	return TRUE;
}


//##########################################
//##########################################
//##########################################
//## SET CONTINUOUS SCAN, NUMBER OF SCANS ##
//##########################################
BOOLEAN doSetNumberOfScans(void)
{

// priority listing:

// if fContinuousScan==TRUE, all other flags ignored
// else if fTimedScan==TRUE, uiScansToDo is ignored and a timed scan is done
//      whichever flag: fScanHours, fScanHours, or fScanHours determines how long
//         scan will go (only one active) and the length corresponds same ui prefix to same name
// if neither fContinuousScan or fTimedScan are set TRUE, then "number of scans" based on uiScansToDo will be done
// Relative variables from "main.h" shown below
//UINT uiScansToDo; 
//BOOLEAN fTimedScan;
//UINT uiScanHours; 
//  BOOLEAN fScanHours;
//UINT uiScanMinutes; 
//  BOOLEAN fScanMinutes;
//UINT uiScanSeconds; 
//  BOOLEAN fScanSeconds; 

	BYTE bChanTemp;

	char tbuff[32]; sprintf(tbuff, "          ");
	curs_set(1);
	echo();
	keypad(win_body, FALSE); 
	while(TRUE)
	{
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Enter 'C' for continuous, 'N' for number of, or 'T' for length of time: ");
		mvwaddstr(win_body, 24, 1, scrnBuff);	
		int cInput = wgetch(win_body);
		char upChar = toupper((char)cInput);

		if(upChar == 'C')
		{
			fContinuousScan = TRUE;
			fTimedScan = FALSE;
			sprintf(scrnBuff, "Will scan until any key is pressed");
			break;
		}
		else if(upChar == 'N') 
		{
			fContinuousScan = FALSE;
			fTimedScan = FALSE;
			fScanHours = FALSE;		
			fScanMinutes = FALSE;	
			fScanSeconds = FALSE;	
			wmove(win_body, 24, 1); wclrtoeol(win_body);
			sprintf(scrnBuff, "Enter number of scans to do (use no commas): "); 
			mvwaddstr(win_body, 24, 1, scrnBuff); 
			wrefresh(win_body);
			sprintf(tbuff, "          ");

			mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
//for troubleshooting
/*
wmove(win_body, 24, 1); wclrtoeol(win_body);
sprintf(scrnBuff, "got string: %s, atoi val: %u", tbuff, atoi(tbuff)); 
mvwaddstr(win_body, 24, 1, scrnBuff); 
wrefresh(win_body);
while(chkKbHit()) doGetch();
while(!chkKbHit()) doGetch();
*/
			uiScansToDo = (UINT)atoi(tbuff);
			if(uiScansToDo == 0) continue;

			while(chkKbHit()) doGetch();  //empty buffer if anything remaining
			sprintf(scrnBuff, "Will scan until %u scans have been returned", uiScansToDo);
				break;
		}
		else if(upChar == 'T')
		{
			fContinuousScan = FALSE;
			fTimedScan = TRUE;

			fScanHours = FALSE;		//just for now 
			fScanMinutes = FALSE;	//just for now 
			fScanSeconds = FALSE;	//just for now 

			wmove(win_body, 24, 1); wclrtoeol(win_body);
			sprintf(scrnBuff, "Enter 'C' (continuous), 'H' (hours), 'M'(minutes), or 'S' (seconds): ");
			mvwaddstr(win_body, 24, 1, scrnBuff);	
			wrefresh(win_body);
			sprintf(tbuff, "          ");
			mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
			wmove(win_body, 24, 1); wclrtoeol(win_body);


			if(strchr(tbuff, 'h') || strchr(tbuff, 'H'))
			{
				fScanHours = TRUE;
				sprintf(scrnBuff, "Enter hours to scan (use no commas): "); 
				mvwaddstr(win_body, 24, 1, scrnBuff); wrefresh(win_body);
				sprintf(tbuff, "          ");
				mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
				uiScanHours = atoi(tbuff); 
				sprintf(scrnBuff, "Will scan %u hours", uiScanHours); 	
				break;
			}
			else if(strchr(tbuff, 'm') || strchr(tbuff, 'M'))
			{
				fScanMinutes = TRUE;
				sprintf(scrnBuff, "Enter minutes to scan (use no commas): "); mvwaddstr(win_body, 24, 1, scrnBuff); 	
				sprintf(tbuff, "          ");
				mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
				uiScanMinutes = atoi(tbuff); 
				sprintf(scrnBuff, "Will scan %u minutes", uiScanMinutes); 	
				break;
			}
			else if(strchr(tbuff, 's') || strchr(tbuff, 'S'))
			{
				fScanSeconds = TRUE;
				sprintf(scrnBuff, "Enter seconds to scan (use no commas): "); mvwaddstr(win_body, 24, 1, scrnBuff); 	
				sprintf(tbuff, "          ");
				mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
				uiScanSeconds = atoi(tbuff); 
				sprintf(scrnBuff, "Will scan %u seconds", uiScanSeconds); 	
				break;
			}
		}
	}
	curs_set(0);
	noecho();
	mvwaddstr(win_body, 24, 1, scrnBuff);
	wrefresh(win_body);
	showScanMenu(); 

	return TRUE;
}

//#############################
//#############################
//#############################
//## GET POLLED MODE VOLTAGE ##
//#############################
BOOLEAN getPolledModeVoltage(void)
{
	if(!fSignedOn) notSignedOnCheck();
	if(!fSignedOn) return FALSE;
	BYTE bChanTemp;

	char tbuff[32]; memset(tbuff, 'x', sizeof(tbuff)); //set to non-zero value
	wmove(win_body, 24, 1); wclrtoeol(win_body);
	wrefresh(win_body);
	sprintf(scrnBuff, "Getting voltage for channel: %u - WAIT", bSingleChanScanChan); 
	mvwaddstr(win_body, 24, 1, scrnBuff);
	bDBG = 1;
	wrefresh(win_body);

	if(bDBG)
	{
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		wclear(win_body);
		curs_set(1); echo(); 
		
		//HIDE CURSES WINDOW
		endwin();
		system ("clear");
	}

	double dblVolts_;
	BYTE bLastDigin_;
	//BYTE getConversion(struct libusb_device_handle *hDevice, USHORT usDevID, 
	//	double* dblVolts_, BYTE* bLastDigin, BYTE bDBG, int* iErr)
	if(getConversion(hDevice_global, usDeviceID, &dblVolts_, &bLastDigin_, bDBG, &iErr))
	{
		if(bDBG)
		{
			sleep(1);
			system ("clear");

			//SHOW CURSES WINDOW
			wrefresh(win_body);

			keypad(win_body, TRUE); 
			curs_set(0);
			wmove(win_body, 24, 1);
			showMainMenu();  //okay here since this is only done from main menu 
			wmove(win_body, 24, 1); wclrtoeol(win_body);
		}
		dblVolts = dblVolts_;
		fGotDigin = TRUE;
		bLastDigin = bLastDigin_;

		curs_set(0);
		noecho();
		keypad(win_body, TRUE); 
		wrefresh(win_body);
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "got: %1.6lf for chan: %d", dblVolts, bSingleChanScanChan);
		mvwaddstr(win_body, 24, 1, scrnBuff);
		wrefresh(win_body);
		showMainMenu(); 
	}
	else sprintf(scrnBuff, "get voltage for chan: %u FAILED", bSingleChanScanChan);

	curs_set(0);
	noecho();
	keypad(win_body, TRUE); 
	return TRUE;
}

//#############################
//#############################
//#############################
//## GET DIGITAL INPUT ########
//#############################
//same as getting a conversion since it's returned in buffer[0] from
// firmware - just quiter - no screen output of conversion which comes
// witth it.
BOOLEAN getDigin(void)
{
	if(!fSignedOn) notSignedOnCheck();
	if(!fSignedOn) return FALSE;

	char tbuff[32]; memset(tbuff, 'x', sizeof(tbuff)); //set to non-zero value
	wmove(win_body, 24, 1); wclrtoeol(win_body);
	wrefresh(win_body);
	sprintf(scrnBuff, "Getting digital input. Wait . . ."); 
	mvwaddstr(win_body, 24, 1, scrnBuff);
	bDBG = 0;
	wrefresh(win_body);

	if(bDBG)
	{
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		wclear(win_body);
		curs_set(1); echo(); 
		
		//HIDE CURSES WINDOW
		endwin();
		system ("clear");
	}

	BYTE bLastDigin_;
	if(getDigitalInput(hDevice_global, usDeviceID, &bLastDigin_, bDBG, &iErr))
	{
		/*
		if(bDBG)
		{
			sleep(1);
			system ("clear");

			//SHOW CURSES WINDOW
			wrefresh(win_body);

			keypad(win_body, TRUE); 
			curs_set(0);
			wmove(win_body, 24, 1);
			showMainMenu();  //okay here since this is only done from main menu 
			wmove(win_body, 24, 1); wclrtoeol(win_body);
		}
		*/
		fGotDigin = TRUE;
		bLastDigin = bLastDigin_;

		curs_set(0);
		noecho();
		keypad(win_body, TRUE); 
		wrefresh(win_body);
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "successfully got digital input: %u", bLastDigin);
		mvwaddstr(win_body, 24, 1, scrnBuff);
		wrefresh(win_body);
		showMainMenu(); 
	}
	else sprintf(scrnBuff, "get voltage for chan: %u FAILED", bSingleChanScanChan);

	curs_set(0);
	noecho();
	keypad(win_body, TRUE); 
	return TRUE;
}

//#############################
//#############################
//#############################
//## RATE CHANGE ##############
//#############################
BOOLEAN doRate(void)
{
	if(!fSignedOn) notSignedOnCheck();
	if(!fSignedOn) return FALSE;
	DOUBLE dblRate_Temp;
	char tbuff[32]; memset(tbuff, 'x', sizeof(tbuff)); //set to non-zero value
	curs_set(1);
	echo();
	keypad(win_body, FALSE); 
	while(TRUE)
	{
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Enter rate 50-1000: ");
		mvwaddstr(win_body, 24, 1, scrnBuff);	
		mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
		dblRate_Temp = (DOUBLE)atof(tbuff);
		if((dblRate_Temp >= 50) && (dblRate_Temp <= 1000)) break;
	}
	sprintf(scrnBuff, "Sending rate: %4.2lf - WAIT", dblRate_Temp);
	mvwaddstr(win_body, 24, 1, scrnBuff);
	wrefresh(win_body);

	DOUBLE dblRateToSend = dblRate_Temp;

	wclear(win_body);
	curs_set(1); echo();

	//HIDE CURSES WINDOW
	endwin();
	system ("clear");
	bDBG = 1;

	//BYTE sendRate(struct libusb_device_handle *hDevice, USHORT usDevID, double* dblRate_, BYTE bDBG, int* iErr);
	bRetVal = sendRate(hDevice_global, usDeviceID, &dblRateToSend, bDBG, &iErr);

	system ("clear");

	//SHOW CURSES WINDOW
	wrefresh(win_body);

	keypad(win_body, TRUE); 
	curs_set(0);
	wmove(win_body, 24, 1);

	if(fPollOrScanMenuShowing == POLL_MENU) showMainMenu();

	wmove(win_body, 24, 1); wclrtoeol(win_body);

	if(!bRetVal)
	{
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Rate send FAILED - try again");
		mvwaddstr(win_body, 24, 1, scrnBuff);
		curs_set(0);
		noecho();
		keypad(win_body, TRUE); 
		return TRUE;
	}
	wmove(win_body, 24, 1); wclrtoeol(win_body);
	if(dblRateToSend != dblRate_Temp)
		sprintf(scrnBuff, "Rate-send success. Rate adjusted to: %4.2lf", dblRateToSend);
	else sprintf(scrnBuff, "Rate %4.3lf send success", dblRateToSend);

	dblRate = dblRateToSend;

	//UPDATE DATA RATE
	if((scanStruct.uiScanToken == CMND_SINGLE_CHAN_SCAN) 
		|| (scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN))
	{ dblDataRate = dblRate; }
	else if((scanStruct.uiScanToken == CMND_MULTI_CHAN_SCAN) 
		|| (scanStruct.uiScanToken == CMND_MULTI_CHAN_DIGIN_SCAN))
	{ dblDataRate = dblRate / (scanStruct.uiScanChans * 5); }
	else if((scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_SCAN) 
		|| (scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN))
	{ dblDataRate = dblRate / ((scanStruct.uiScanChans + 1) * 5); }

	mvwaddstr(win_body, 24, 1, scrnBuff);
	curs_set(0);
	noecho();
	keypad(win_body, TRUE); 

	if(fPollOrScanMenuShowing != POLL_MENU) showScanMenu();
	else showMainMenu(); //4-13-17

	return TRUE;
}

//###############################
//###############################
//###############################
//## INIT / SIGNON-ON ###########
//###############################
BOOLEAN chkDevID(USHORT* usDevID)
{
	USHORT usTempDevID_Local = 0;
	BOOLEAN fGotDevID = FALSE;
	char tbuff[32]; memset(tbuff, 'x', sizeof(tbuff)); //init buff to anything except 0
	curs_set(1);
	echo();
	keypad(win_body, FALSE); 
	wrefresh(win_body);

	while(chkKbHit() == TRUE) { usleep(10); doGetch();} 

	wmove(win_body, 24, 1); wclrtoeol(win_body);
	sprintf(scrnBuff, "Enter Device ID: ");
	mvwaddstr(win_body, 24, 1, scrnBuff);	
	mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
	usTempDevID_Local = atoi(tbuff);
	if(!usTempDevID_Local)
	{
		while(chkKbHit() == TRUE) { usleep(10); doGetch();} 
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Enter Device ID (or 0 to cancel): ");
		mvwaddstr(win_body, 24, 1, scrnBuff);
		memset(tbuff, 'x', sizeof(tbuff));
		mvwgetnstr(win_body, 24, strlen(scrnBuff) + 1, tbuff, strlen(tbuff));
		usTempDevID_Local = atoi(tbuff);
		if(usTempDevID_Local) { fGotDevID = TRUE; *usDevID = usTempDevID_Local; }
		else { fGotDevID = FALSE; }
	}
	else
	{ fGotDevID = TRUE; }

	wmove(win_body, 24, 1); wclrtoeol(win_body);
	sprintf(scrnBuff, "you entered: %u ", usTempDevID_Local);
	mvwaddstr(win_body, 24, 1, scrnBuff);	
	wrefresh(win_body);
	keypad(win_body, TRUE); 
	noecho();
	curs_set(0);
	wrefresh(win_body);
	if(!fGotDevID) return FALSE;

	BOOLEAN fRetValMainInit = FALSE;
	fRetValMainInit = initUsbLib(&usTempDevID_Local, FALSE);

	if(fRetValMainInit) 
	{
		usDeviceID = usTempDevID_Local;
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Signing on: %u - WAIT", usDeviceID);
		mvwaddstr(win_body, 24, 1, scrnBuff);	
		wrefresh(win_body);
		curs_set(1); echo(); 

		//HIDE CURSES WINDOW		
		endwin();
		system ("clear");
		bDBG = 1;
		printf("Found and connected to device: %u\n", usDeviceID);
		bRetVal = signonFull(hDevice_global, usDeviceID, bMainChan, &dblRate, bDBG, &iErr);

		//NOTE: doesn't seem to happen after device disconnect/reconnect
		if(!bRetVal)
		{
			printf("\nsignonFull failed for device: %u\n", usDeviceID);
			printf("any key to continue\n");
			while(!chkKbHit()) usleep(10);
			doGetch();
		}

		system ("clear");
		
		//SHOW CURSES WINDOW AGAIN		
		wrefresh(win_body); //4-5-17

		keypad(win_body, TRUE); 
		curs_set(0);
		wmove(win_body, 24, 1);

		if(fPollOrScanMenuShowing == POLL_MENU) showMainMenu();

		wmove(win_body, 24, 1); wclrtoeol(win_body);
		if(bRetVal)
		{ 
			fSignedOn = TRUE;
			sprintf(scrnBuff, "Found and signed-on Device: %u", usDeviceID);
			mvwaddstr(win_body, 24, 1, scrnBuff);	
			wrefresh(win_body);
		}
		else
		{
			fSignedOn = FALSE;
			sprintf(scrnBuff, "Found but failed to signon Device: %u", usDeviceID);
			mvwaddstr(win_body, 24, 1, scrnBuff);	
			wrefresh(win_body);
		}

		sprintf(scrnBuff, "D) Set/Change Device ID: %4u            ", usDeviceID);
		putMainScrn(3, scrnBuff);

		wrefresh(win_body);
	}
	else
	{
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Failed finding device: %u", usTempDevID_Local);
		mvwaddstr(win_body, 24, 1, scrnBuff);	
		wrefresh(win_body);
		return FALSE;
	}
	curs_set(0);
	noecho();
	keypad(win_body, TRUE); 

	//NOTE; always back to main menu when this is called
	showMainMenu(); //4-5-17
	return TRUE;
}

//########################
//########################
//########################
//## SHOW MAIN MENU ######
//########################
BOOLEAN showMainMenu(void)
{
	wclear(win_body);
	wrefresh(win_body);

	fPollOrScanMenuShowing = POLL_MENU;

	sprintf(scrnBuff, "Model 30x Data Acquisition system");
	putMainScrn(0, scrnBuff);
	sprintf(scrnBuff, "setup and polled mode screen");
	putMainScrn(1, scrnBuff);
	wmove(win_body, 3, 0); wclrtoeol(win_body);
	if(!usDeviceID)
		sprintf(scrnBuff, "D) Set/Change Device ID  (must do first)");
	else
		sprintf(scrnBuff, "D) Set/Change Device ID: %4u            ", usDeviceID);
	putMainScrn(3, scrnBuff);

	wmove(win_body, 5, 1); wclrtoeol(win_body);
	sprintf(scrnBuff, "R) Enter a rate (current: %4.2lf)        ", dblRate);
	if(dblRate >= 1000) //for alignment on menu screen
		mvwaddstr(win_body, 5, ((80 / 2) - (strlen(scrnBuff) / 2)) + 1, scrnBuff);  
	else if(dblRate < 1000) //for alignment on menu screen
		mvwaddstr(win_body, 5, ((80 / 2) - (strlen(scrnBuff) / 2)), scrnBuff);
	else if (dblRate < 100)
		mvwaddstr(win_body, 5, ((80 / 2) - (strlen(scrnBuff) / 2)) + 1, scrnBuff);
 
	wmove(win_body, 7, 0); wclrtoeol(win_body);
	sprintf(scrnBuff, "C) Enter a channel (current: %d)         ", bSingleChanScanChan);
	putMainScrn(7, scrnBuff);

	wmove(win_body, 9, 0); wclrtoeol(win_body);
	if(!fDacSet)
	{
		sprintf(scrnBuff, "A) Set a DAC voltage                    ");
		putMainScrn(9, scrnBuff);
	}
	else
	{
		wmove(win_body, 9, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "A) Set DAC voltage (last: DAC:%d volts: %1.3lf) ", bLastDAC, dblLastDAC_Val);
		mvwaddstr(win_body, 9, ((80 / 2) - (strlen(scrnBuff) / 2)) + 3, scrnBuff);
	}

	wmove(win_body, 11, 0); wclrtoeol(win_body);
	if(!fDigoutSent)
	{
		sprintf(scrnBuff, "O) Send a digital output (none yet)      ");
		putMainScrn(11, scrnBuff);
	}
	else
	{
		sprintf(scrnBuff, "O) Send a digital output (last: %3d)    ", bLastDigout); 
		putMainScrn(11, scrnBuff);
	}

	wmove(win_body, 13, 0); wclrtoeol(win_body);
	if(!fGotDigin)
	{
		sprintf(scrnBuff, "I) Get a digital input (not yet read)   ");
		putMainScrn(13, scrnBuff);
	}
	else
	{
		sprintf(scrnBuff, "I) Get a digital input (last read: %d)   ", bLastDigin);
		if(bLastDigin >= 100) mvwaddstr(win_body, 13, ((80 / 2) - (strlen(scrnBuff) / 2)) + 1, scrnBuff);
		else mvwaddstr(win_body, 13, ((80 / 2) - (strlen(scrnBuff) / 2)) , scrnBuff);
	}
	wrefresh(win_body);
	
	wmove(win_body, 15, 0); wclrtoeol(win_body);
	if(!fSignedOn)
	{
		sprintf(scrnBuff, "G) Get a voltage                        ");
		putMainScrn(15, scrnBuff);
	}
	else
	{
		sprintf(scrnBuff, "G) Get a voltage (last: %01.6lf for chan: %d)", dblVolts, bSingleChanScanChan); 
		if(dblVolts < 0) //for alignment on menu screen
			mvwaddstr(win_body, 15, ((80 / 2) - (strlen(scrnBuff) / 2)) + 3, scrnBuff);
		else
			mvwaddstr(win_body, 15, ((80 / 2) - (strlen(scrnBuff) / 2)) + 2, scrnBuff);
	}


	wmove(win_body, 17, 0); wclrtoeol(win_body);
	sprintf(scrnBuff, "M) Misc - offset/fullScale/system calibration   ");
	mvwaddstr(win_body, 17, ((80 / 2) - (strlen(scrnBuff) / 2)) + 4, scrnBuff);

	wmove(win_body, 19, 0); wclrtoeol(win_body);
	sprintf(scrnBuff, "S) Scan menu                             ");
	putMainScrn(19, scrnBuff);

	wmove(win_body, 21, 0); wclrtoeol(win_body);
	sprintf(scrnBuff, "Q) Quit                                 ");
	putMainScrn(21, scrnBuff);
	wrefresh(win_body);
}

//########################
//########################
//########################
//## SHOW SCAN MENU ######
//########################
BOOLEAN showScanMenu(void)
{
	fPollOrScanMenuShowing = SCAN_MENU;
	wmove(win_body, 24, 1); wclrtoeol(win_body);

	int cInput;
	sprintf(scrnBuff, "Model 30x Data Acquisition system");
	putMainScrn(0, scrnBuff);
	sprintf(scrnBuff, "Scan mode screen");
	putMainScrn(1, scrnBuff);

	if(!fCurrentScanTypeChanged)
	{
		sprintf(scrnBuff, "T) Scan type (currently single chan)          ");
		putMainScrn(3, scrnBuff);
	}
	else
	{
		switch(scanStruct.uiScanToken)
		{
			case CMND_SINGLE_CHAN_SCAN:
				sprintf(scrnBuff, "T) Scan type (currently single-chan)          ");
				putMainScrn(3, scrnBuff); break;
			case CMND_SINGLE_CHAN_DIGIN_SCAN:
				sprintf(scrnBuff, "T) Scan type (currently single-chan dig-in)   ");
				putMainScrn(3, scrnBuff); break;
			case CMND_MULTI_CHAN_SCAN:
				sprintf(scrnBuff, "T) Scan type (currently multi-chan)           ");
				putMainScrn(3, scrnBuff); break;
			case CMND_MULTI_CHAN_DIGIN_SCAN:
				sprintf(scrnBuff, "T) Scan type (currently multi-chan dig-in)    ");
				putMainScrn(3, scrnBuff); break;
			case CMND_MULTI_CHAN_CAL_SCAN:
				sprintf(scrnBuff, "T) Scan type (currently multi-chan cal)       ");
				putMainScrn(3, scrnBuff); break;
			case CMND_MULTI_CHAN_CAL_DIGIN_SCAN:
				sprintf(scrnBuff, "T) Scan type (currently multi-chan cal dig-in)");
				putMainScrn(3, scrnBuff); break;
		}
	}

	if((scanStruct.uiScanToken == CMND_SINGLE_CHAN_SCAN) 
		|| (scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN))
		sprintf(scrnBuff, "C) Set chan (currently chan: %u)                ", bSingleChanScanChan);
	else
		sprintf(scrnBuff, "C) Set chan cnt (currently %u)                  ", scanStruct.uiScanChans);
	putMainScrn(5, scrnBuff);


	sprintf(scrnBuff, "note: lines may wrap in multi-chan");
	wmove(win_body, 6, 1); wclrtoeol(win_body);
	mvwaddstr(win_body, 6, 21, scrnBuff);	
	sprintf(scrnBuff, "      scan if many chans selected");
	wmove(win_body, 7, 1); wclrtoeol(win_body);
	mvwaddstr(win_body, 7, 21, scrnBuff);	

	wmove(win_body, 9, 1); wclrtoeol(win_body);
	wmove(win_body, 10, 1); wclrtoeol(win_body);
	sprintf(scrnBuff, "N) Set number of scans to continuous          ");
	putMainScrn(9, scrnBuff);
	if(fContinuousScan)
		sprintf(scrnBuff, "  or limited (currently continuous)             ");
	else
	{
/* just for reference
BOOLEAN fTimedScan;
UINT uiScanHours; 
UINT uiScanMinutes; 
UINT uiScanSeconds; 
BOOLEAN fScanHours;
UINT uiScanMinutes; 
BOOLEAN fScanMinutes;
UINT uiScanSeconds; 
BOOLEAN fScanSeconds; 
*/
		if(!fTimedScan) sprintf(scrnBuff, "  or limited (currently %5u scans)   ", uiScansToDo);
		else
		{
			if(fScanHours) sprintf(scrnBuff, "  or limited (currently %2u hours)    ", uiScanHours);
			else if(fScanMinutes) sprintf(scrnBuff, "  or limited (currently %2u minutes)    ", uiScanMinutes);
			else if(fScanSeconds) sprintf(scrnBuff, "  or limited (currently %2u seconds)    ", uiScanSeconds);
		}
	}
	putMainScrn(10, scrnBuff);

	//this allows for low-level stacking of scans which allows for slightly more 
	// time for user-type apps to have a time-slice without buffer in firmware
	// wrapping. In other words, the kernel-mode threading process can be reading 
	// data from the firmware while user mode apps are doing things and then the 
	// app can start processing that data when it gets a slice. Set it here since
	// this is updated whenever the rate is updated.

	//following should work with all scan type scenarios, if there is a microcode buffer
	// wrap problem on a particular system. HOWEVER having only 2 stacked seems to work fine
	// with all scan type scenarios. So use it unless there is a problem.
//	scanStruct.uiStackedScans = (UINT)dblDataRate / 20;  //so since 50@1000 works, do this
//	if(scanStruct.uiStackedScans < 1) scanStruct.uiStackedScans = 1; 

	//following (instead of lines above) "should" work on most systems along with the queueing
	// of scan requests as is implemented in helperFuncs.c with queued quantity set by MAX_QUEUE
	scanStruct.uiStackedScans = 2;

	wmove(win_body, 12, 1); wclrtoeol(win_body);
	if(dblRate < 1005)
		sprintf(scrnBuff, "R) Set internal rate (currently %3.2lf)       ", dblRate);
	if(dblRate < 1000)
		sprintf(scrnBuff, "R) Set internal rate (currently %3.2lf)        ", dblRate);
	if(dblRate < 100)
		sprintf(scrnBuff, "R) Set internal rate (currently %3.2lf)        ", dblRate); 
	//never less than double digit for internal data rate with min of 60Hz
	putMainScrn(12, scrnBuff);

	wmove(win_body, 13, 1); wclrtoeol(win_body);
	if(dblDataRate < 1005)
		sprintf(scrnBuff, "      scan data rate currently %3.2lf              ", dblDataRate);
	if(dblDataRate < 1000)
		sprintf(scrnBuff, "      scan rate currently %3.2lf               ", dblDataRate);
	if(dblDataRate < 100)
		sprintf(scrnBuff, "      scan rate currently %3.2lf               ", dblDataRate);
	if(dblDataRate < 10)
		sprintf(scrnBuff, "      scan rate currently %3.2lf                ", dblDataRate);
	putMainScrn(13, scrnBuff);

	if(fDoScanLog)
	{
		wmove(win_body, 15, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "D) Data display and/or log to file [datestamp.txt] (now file only)  ");
		mvwaddstr(win_body, 15, ((80 / 2) - (strlen(scrnBuff) / 2)) + 11, scrnBuff);
	}
	if(fDoDispData) 
	{
		wmove(win_body, 15, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "D) Data display and/or log to file (now display only)  ");
		mvwaddstr(win_body, 15, ((80 / 2) - (strlen(scrnBuff) / 2)) + 4, scrnBuff);
	}
	if(fDoScanLog && fDoDispData)
	{
		wmove(win_body, 15, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "D) Data display and/or log to file [datestamp.txt] (now file and display)  ");
		mvwaddstr(win_body, 15, ((80 / 2) - (strlen(scrnBuff) / 2)) + 14, scrnBuff);
	}
	// maybe add as option just for testing?
	if(!fDoScanLog && !fDoDispData)
	{
		wmove(win_body, 15, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "D) Data display and/or log to file (no log file and no display) ");
		mvwaddstr(win_body, 15, ((80 / 2) - (strlen(scrnBuff) / 2)) + 10, scrnBuff);
	}
	
	sprintf(scrnBuff, "S) Start scan                                 ");
	putMainScrn(17, scrnBuff);
	sprintf(scrnBuff, "E) Exit scan screen                           ");
	putMainScrn(19, scrnBuff);
	wrefresh(win_body);
	curs_set(0);
	wmove(win_body, 24, 1);
	wrefresh(win_body);
	noecho();	//no echo key input

	return TRUE;
}

//############################
//############################
//############################
//## CHECK FOR SIGNON ########
//## DO IF NOT        ########
//############################
BOOLEAN notSignedOnCheck()
{
	while(!fSignedOn)
	{
		USHORT usDevTemp = 0;
		curs_set(1);
		echo();
		keypad(win_body, FALSE); 
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		sprintf(scrnBuff, "Not signed on. Signon now? (Y)es (N)o ");
		mvwaddstr(win_body, 24, 1, scrnBuff);	
		int cInput = wgetch(win_body);
		char upChar = toupper((char)cInput);
		wmove(win_body, 24, 1); wclrtoeol(win_body);
		if(upChar == 'N')
		{
			wmove(win_body, 24, 1); wclrtoeol(win_body);
			sprintf(scrnBuff, "Need to signon to do anything");
			mvwaddstr(win_body, 24, 1, scrnBuff);
			curs_set(0);
			noecho();
			keypad(win_body, TRUE); 
			return FALSE;	
		}
		if(!chkDevID(&usDevTemp)) return FALSE;

		fSignedOn = TRUE;

		showMainMenu(); //4-5-17
		break;
	}
	return TRUE;
}

//#######################
//#######################
//#######################
//## CHECK FOR KEY HIT ##
//#######################
//NOTE: this function also checks to see if device is still connected.
int kbhit(BOOLEAN fIncDec)
{
	while(true)
	{
		USHORT usDevID_SAVE = usDeviceID;
		if(usDeviceID)
		{
			#define NO_CONNECT_ERROR 0
			#define PWR_CONNECT_ERR 1
			#define USB_CONNECT_ERR 2
			#define UNKNOWN_CONNECT_ERR 3
			USHORT usDevID_TEMP = 0;
			UINT uiErrType = 0;

			if(!checkDevConnect(hDevice_global, &uiErrType))
			{
				usDeviceID = 0;
				showMainMenu();
				//4-24-17e else showScanMenu();
				if((uiErrType & UNKNOWN_CONNECT_ERR) == UNKNOWN_CONNECT_ERR)
					sprintf(scrnBuff, "Device %u unknown error", usDevID_SAVE);
				else if((uiErrType & USB_CONNECT_ERR) == USB_CONNECT_ERR)
					sprintf(scrnBuff, "Device %u disconnected", usDevID_SAVE);
				else if((uiErrType & PWR_CONNECT_ERR) == PWR_CONNECT_ERR)
					sprintf(scrnBuff, "Device %u seems to have lost power", usDevID_SAVE);
				else if(uiErrType == 0)
					sprintf(scrnBuff, "Device %u unknown error", usDevID_SAVE);
				wmove(win_body, 23, 1); wclrtoeol(win_body);
				mvwaddstr(win_body,  23, 1, scrnBuff);

				sprintf(scrnBuff, "Fix then  \"D) set/change Device ID\"  above to reinitialize");
				wmove(win_body, 24, 1); wclrtoeol(win_body);
				mvwaddstr(win_body, 24, 1, scrnBuff);
				wrefresh(win_body);
			}
			else
			{
				wmove(win_body, 23, 1); wclrtoeol(win_body); //clear disconnect message
				wmove(win_body, 24, 1); wclrtoeol(win_body);
			}
		}

		if(chkKbHit())
		{
			int ch = wgetch(win_body);
			if(((ch < 48) || (ch > 90)) && ((ch < 97) || (ch > 122)) && ((fIncDec == TRUE) && (ch != 46)) ) continue;
			if(ch != ERR)
			{ ungetch(ch); return 1; }
			else return 0;
		}
	}
}


BOOLEAN cleanupForExit(void)
{
}

//###################################
//###################################
//###################################
//## CENTER MENU TEXT ###############
//###################################
//used to center menu items on screen. Due to length of some menu 
// items, the following doesn't work for all, but does for most.
BOOLEAN putMainScrn(int y, char* str)
{
	mvwaddstr(win_body,  y, (80 / 2) - (strlen(str) / 2), str);
}

